/** ****************************************************************************
 * @file     Stopwatch.cpp
 * @brief    Stopwatch measure time of execution in microseconds
 *
 * @note
 * Copyright ZUK ELZAB S.A. All rights reserved. @n
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ELZAB S.A.
 * The copyright notice above does not evidence any actual or intended
 * publication of such source code.
 *
 *******************************************************************************/
#include "stopwatch.h"
#include <string.h>

//#pragma GCC push_options
//#pragma GCC optimize ("O0")
/** ****************************************************************************
 * @brief Constructor
 *******************************************************************************/
cStopwatch::cStopwatch()
{
  ENABLE_DEMCR();           // enable DWT hardware
  UNLOCK_DWT();
  __DSB();
  ENABLE_CYCLE_COUNTER();   // start counting
  __DSB();
  //RESET_CYCLE_COUNTER();    // reset cycle counter
  cycles = GET_CYCLE_COUNTER();

}
//#pragma GCC pop_options

//temporary reference from internet (for comparison only):
//#define RESET_CYCLE_COUNTER()  do { \
//        CoreDebug->DEMCR = CoreDebug_DEMCR_TRCENA_Msk; \
//        __DSB(); DWT->LAR = 0xC5ACCE55; __DSB(); \
//        DWT->CTRL &= ~DWT_CTRL_CYCCNTENA_Msk; \
//        DWT->CYCCNT = 0; \
//        DWT->CTRL = DWT_CTRL_CYCCNTENA_Msk; \
//    }while(0)
//#define GET_CYCLE_COUNTER(x)                x=DWT->CYCCNT;


/** ****************************************************************************
 * @brief Destructor
 *******************************************************************************/
cStopwatch::~cStopwatch()
{
  //UNLOCK_DWT();
  //DISABLE_CYCLE_COUNTER(); // disable counting if not used any more
}

/** ****************************************************************************
 * @brief Start counting
 *******************************************************************************/
void cStopwatch::Start()
{
  //UNLOCK_DWT();
  cycles = GET_CYCLE_COUNTER(); //RESET_CYCLE_COUNTER(); // reset cycle counter
}

/** *****************************************************************************
 * @brief  Get time of execution (from last Start() or last GetTime())
 *
 * @note It count in correct way only the time from range 0..19'884'107us
 *
 * @return time of execution in [us]
 *******************************************************************************/
u32 cStopwatch::GetTime()
{
  // get cycle counter
  u32 cycles2us = GET_CYCLE_COUNTER();    // in [command cycle]

  if ( cycles2us > cycles )
    cycles2us -= cycles;
  else
    cycles2us += ((0xFFFFFFFF - cycles) + 1);

  cycles2us /= SYSTEM_CORE_CLOCK_IN_MHZ;  // in [us]
  //UNLOCK_DWT();
  cycles = GET_CYCLE_COUNTER(); //RESET_CYCLE_COUNTER();  // reset cycle counter
  return cycles2us;
}

// -----------------------------------------------------------------------------
// End of file -----------------------------------------------------------------
// -----------------------------------------------------------------------------
